@@ -47,9 +47,10 @@ from moto.config.exceptions import (
 
 from moto.core import BaseBackend, BaseModel
 from moto.s3.config import s3_account_public_access_block_query, s3_config_query
-
 from moto.core import ACCOUNT_ID as DEFAULT_ACCOUNT_ID
 
+from moto.iam.config import role_config_query, policy_config_query
+
 POP_STRINGS = [
     "capitalizeStart",
     "CapitalizeStart",
@@ -64,6 +65,8 @@ DEFAULT_PAGE_SIZE = 100
 RESOURCE_MAP = {
     "AWS::S3::Bucket": s3_config_query,
     "AWS::S3::AccountPublicAccessBlock": s3_account_public_access_block_query,
+    "AWS::IAM::Role": role_config_query,
+    "AWS::IAM::Policy": policy_config_query,
 }
 
 
@@ -977,6 +980,7 @@ class ConfigBackend(BaseBackend):
                 limit,
                 next_token,
                 resource_region=resource_region,
+                aggregator=self.config_aggregators.get(aggregator_name).__dict__,
             )
 
         resource_identifiers = []
@@ -987,7 +991,6 @@ class ConfigBackend(BaseBackend):
                 "ResourceType": identifier["type"],
                 "ResourceId": identifier["id"],
             }
-
             if identifier.get("name"):
                 item["ResourceName"] = identifier["name"]
 
@@ -1003,9 +1006,9 @@ class ConfigBackend(BaseBackend):
     def get_resource_config_history(self, resource_type, id, backend_region):
         """Returns the configuration of an item in the AWS Config format of the resource for the current regional backend.
 
-            NOTE: This is --NOT-- returning history as it is not supported in moto at this time. (PR's welcome!)
-                  As such, the later_time, earlier_time, limit, and next_token are ignored as this will only
-                  return 1 item. (If no items, it raises an exception)
+        NOTE: This is --NOT-- returning history as it is not supported in moto at this time. (PR's welcome!)
+              As such, the later_time, earlier_time, limit, and next_token are ignored as this will only
+              return 1 item. (If no items, it raises an exception)
         """
         # If the type isn't implemented then we won't find the item:
         if resource_type not in RESOURCE_MAP:
@@ -1087,10 +1090,10 @@ class ConfigBackend(BaseBackend):
     ):
         """Returns the configuration of an item in the AWS Config format of the resource for the current regional backend.
 
-            As far a moto goes -- the only real difference between this function and the `batch_get_resource_config` function is that
-            this will require a Config Aggregator be set up a priori and can search based on resource regions.
+        As far a moto goes -- the only real difference between this function and the `batch_get_resource_config` function is that
+        this will require a Config Aggregator be set up a priori and can search based on resource regions.
 
-            Note: moto will IGNORE the resource account ID in the search query.
+        Note: moto will IGNORE the resource account ID in the search query.
         """
         if not self.config_aggregators.get(aggregator_name):
             raise NoSuchConfigurationAggregatorException()
