@@ -1,23 +1,25 @@
 from __future__ import unicode_literals
+
 import re
 import uuid
+from copy import copy
 from datetime import datetime
 from random import random, randint
 
 import pytz
 from boto3 import Session
 
+from moto.core import BaseBackend, BaseModel, CloudFormationModel, ACCOUNT_ID
 from moto.core.exceptions import JsonRESTError
-from moto.core import BaseBackend, BaseModel, CloudFormationModel
-from moto.core.utils import unix_time
+from moto.core.utils import unix_time, pascal_to_camelcase, remap_nested_keys
 from moto.ec2 import ec2_backends
-from copy import copy
-
 from .exceptions import (
     ServiceNotFoundException,
     TaskDefinitionNotFoundException,
     TaskSetNotFoundException,
     ClusterNotFoundException,
+    InvalidParameterException,
+    RevisionNotFoundException,
 )
 
 
@@ -47,8 +49,8 @@ class BaseObject(BaseModel):
 class Cluster(BaseObject, CloudFormationModel):
     def __init__(self, cluster_name, region_name):
         self.active_services_count = 0
-        self.arn = "arn:aws:ecs:{0}:012345678910:cluster/{1}".format(
-            region_name, cluster_name
+        self.arn = "arn:aws:ecs:{0}:{1}:cluster/{2}".format(
+            region_name, ACCOUNT_ID, cluster_name
         )
         self.name = cluster_name
         self.pending_tasks_count = 0
@@ -124,32 +126,80 @@ class TaskDefinition(BaseObject, CloudFormationModel):
         volumes=None,
         tags=None,
         placement_constraints=None,
+        requires_compatibilities=None,
+        cpu=None,
+        memory=None,
+        task_role_arn=None,
+        execution_role_arn=None,
     ):
         self.family = family
         self.revision = revision
-        self.arn = "arn:aws:ecs:{0}:012345678910:task-definition/{1}:{2}".format(
-            region_name, family, revision
+        self.arn = "arn:aws:ecs:{0}:{1}:task-definition/{2}:{3}".format(
+            region_name, ACCOUNT_ID, family, revision
         )
-        self.container_definitions = container_definitions
+
+        default_container_definition = {
+            "cpu": 0,
+            "portMappings": [],
+            "essential": True,
+            "environment": [],
+            "mountPoints": [],
+            "volumesFrom": [],
+        }
+        self.container_definitions = []
+        for container_definition in container_definitions:
+            full_definition = default_container_definition.copy()
+            full_definition.update(container_definition)
+            self.container_definitions.append(full_definition)
+
         self.tags = tags if tags is not None else []
+
         if volumes is None:
             self.volumes = []
         else:
             self.volumes = volumes
-        if network_mode is None:
+
+        if not requires_compatibilities or requires_compatibilities == ["EC2"]:
+            self.compatibilities = ["EC2"]
+        else:
+            self.compatibilities = ["EC2", "FARGATE"]
+
+        if network_mode is None and "FARGATE" not in self.compatibilities:
             self.network_mode = "bridge"
+        elif "FARGATE" in self.compatibilities:
+            self.network_mode = "awsvpc"
         else:
             self.network_mode = network_mode
+
+        if task_role_arn is not None:
+            self.task_role_arn = task_role_arn
+        if execution_role_arn is not None:
+            self.execution_role_arn = execution_role_arn
+
         self.placement_constraints = (
             placement_constraints if placement_constraints is not None else []
         )
 
+        self.requires_compatibilities = requires_compatibilities
+
+        self.cpu = cpu
+        self.memory = memory
+        self.status = "ACTIVE"
+
     @property
     def response_object(self):
         response_object = self.gen_response_object()
         response_object["taskDefinitionArn"] = response_object["arn"]
         del response_object["arn"]
         del response_object["tags"]
+
+        if not response_object["requiresCompatibilities"]:
+            del response_object["requiresCompatibilities"]
+        if not response_object["cpu"]:
+            del response_object["cpu"]
+        if not response_object["memory"]:
+            del response_object["memory"]
+
         return response_object
 
     @property
@@ -174,8 +224,10 @@ class TaskDefinition(BaseObject, CloudFormationModel):
         family = properties.get(
             "Family", "task-definition-{0}".format(int(random() * 10 ** 6))
         )
-        container_definitions = properties["ContainerDefinitions"]
-        volumes = properties.get("Volumes")
+        container_definitions = remap_nested_keys(
+            properties.get("ContainerDefinitions", []), pascal_to_camelcase
+        )
+        volumes = remap_nested_keys(properties.get("Volumes", []), pascal_to_camelcase)
 
         ecs_backend = ecs_backends[region_name]
         return ecs_backend.register_task_definition(
@@ -220,10 +272,11 @@ class Task(BaseObject):
         resource_requirements,
         overrides={},
         started_by="",
+        tags=[],
     ):
         self.cluster_arn = cluster.arn
-        self.task_arn = "arn:aws:ecs:{0}:012345678910:task/{1}".format(
-            cluster.region_name, str(uuid.uuid4())
+        self.task_arn = "arn:aws:ecs:{0}:{1}:task/{2}".format(
+            cluster.region_name, ACCOUNT_ID, str(uuid.uuid4())
         )
         self.container_instance_arn = container_instance_arn
         self.last_status = "RUNNING"
@@ -232,6 +285,7 @@ class Task(BaseObject):
         self.overrides = overrides
         self.containers = []
         self.started_by = started_by
+        self.tags = tags
         self.stopped_reason = ""
         self.resource_requirements = resource_requirements
 
@@ -252,10 +306,11 @@ class Service(BaseObject, CloudFormationModel):
         scheduling_strategy=None,
         tags=None,
         deployment_controller=None,
+        launch_type=None,
     ):
         self.cluster_arn = cluster.arn
-        self.arn = "arn:aws:ecs:{0}:012345678910:service/{1}".format(
-            cluster.region_name, service_name
+        self.arn = "arn:aws:ecs:{0}:{1}:service/{2}".format(
+            cluster.region_name, ACCOUNT_ID, service_name
         )
         self.name = service_name
         self.status = "ACTIVE"
@@ -268,12 +323,14 @@ class Service(BaseObject, CloudFormationModel):
         self.task_sets = []
         self.deployment_controller = deployment_controller or {"type": "ECS"}
         self.events = []
+        self.launch_type = launch_type
         if self.deployment_controller["type"] == "ECS":
             self.deployments = [
                 {
                     "createdAt": datetime.now(pytz.utc),
                     "desiredCount": self.desired_count,
                     "id": "ecs-svc/{}".format(randint(0, 32 ** 12)),
+                    "launchType": self.launch_type,
                     "pendingCount": self.desired_count,
                     "runningCount": 0,
                     "status": "PRIMARY",
@@ -429,8 +486,8 @@ class ContainerInstance(BaseObject):
                 "type": "STRINGSET",
             },
         ]
-        self.container_instance_arn = "arn:aws:ecs:{0}:012345678910:container-instance/{1}".format(
-            region_name, str(uuid.uuid4())
+        self.container_instance_arn = "arn:aws:ecs:{0}:{1}:container-instance/{2}".format(
+            region_name, ACCOUNT_ID, str(uuid.uuid4())
         )
         self.pending_tasks_count = 0
         self.remaining_resources = [
@@ -481,6 +538,7 @@ class ContainerInstance(BaseObject):
             if ec2_instance.platform == "windows"
             else "linux",  # options are windows and linux, linux is default
         }
+        self.registered_at = datetime.now(pytz.utc)
 
     @property
     def response_object(self):
@@ -489,6 +547,10 @@ class ContainerInstance(BaseObject):
             self._format_attribute(name, value)
             for name, value in response_object["attributes"].items()
         ]
+        if isinstance(response_object["registeredAt"], datetime):
+            response_object["registeredAt"] = unix_time(
+                response_object["registeredAt"].replace(tzinfo=None)
+            )
         return response_object
 
     def _format_attribute(self, name, value):
@@ -501,8 +563,8 @@ class ContainerInstance(BaseObject):
 class ClusterFailure(BaseObject):
     def __init__(self, reason, cluster_name, region_name):
         self.reason = reason
-        self.arn = "arn:aws:ecs:{0}:012345678910:cluster/{1}".format(
-            region_name, cluster_name
+        self.arn = "arn:aws:ecs:{0}:{1}:cluster/{2}".format(
+            region_name, ACCOUNT_ID, cluster_name
         )
 
     @property
@@ -516,8 +578,8 @@ class ClusterFailure(BaseObject):
 class ContainerInstanceFailure(BaseObject):
     def __init__(self, reason, container_instance_id, region_name):
         self.reason = reason
-        self.arn = "arn:aws:ecs:{0}:012345678910:container-instance/{1}".format(
-            region_name, container_instance_id
+        self.arn = "arn:aws:ecs:{0}:{1}:container-instance/{2}".format(
+            region_name, ACCOUNT_ID, container_instance_id
         )
 
     @property
@@ -571,8 +633,8 @@ class TaskSet(BaseObject):
 
         cluster_name = self.cluster.split("/")[-1]
         service_name = self.service.split("/")[-1]
-        self.task_set_arn = "arn:aws:ecs:{0}:012345678910:task-set/{1}/{2}/{3}".format(
-            region_name, cluster_name, service_name, self.id
+        self.task_set_arn = "arn:aws:ecs:{0}:{1}:task-set/{2}/{3}/{4}".format(
+            region_name, ACCOUNT_ID, cluster_name, service_name, self.id
         )
 
     @property
@@ -611,6 +673,16 @@ class EC2ContainerServiceBackend(BaseBackend):
         self.__dict__ = {}
         self.__init__(region_name)
 
+    def _get_cluster(self, name):
+        # short name or full ARN of the cluster
+        cluster_name = name.split("/")[-1]
+
+        cluster = self.clusters.get(cluster_name)
+        if not cluster:
+            raise ClusterNotFoundException
+
+        return cluster
+
     def describe_task_definition(self, task_definition_str):
         task_definition_name = task_definition_str.split("/")[-1]
         if ":" in task_definition_name:
@@ -657,11 +729,9 @@ class EC2ContainerServiceBackend(BaseBackend):
         return list_clusters, failures
 
     def delete_cluster(self, cluster_str):
-        cluster_name = cluster_str.split("/")[-1]
-        if cluster_name in self.clusters:
-            return self.clusters.pop(cluster_name)
-        else:
-            raise Exception("{0} is not a cluster".format(cluster_name))
+        cluster = self._get_cluster(cluster_str)
+
+        return self.clusters.pop(cluster.name)
 
     def register_task_definition(
         self,
@@ -671,6 +741,11 @@ class EC2ContainerServiceBackend(BaseBackend):
         network_mode=None,
         tags=None,
         placement_constraints=None,
+        requires_compatibilities=None,
+        cpu=None,
+        memory=None,
+        task_role_arn=None,
+        execution_role_arn=None,
     ):
         if family in self.task_definitions:
             last_id = self._get_last_task_definition_revision_id(family)
@@ -687,6 +762,11 @@ class EC2ContainerServiceBackend(BaseBackend):
             network_mode=network_mode,
             tags=tags,
             placement_constraints=placement_constraints,
+            requires_compatibilities=requires_compatibilities,
+            cpu=cpu,
+            memory=memory,
+            task_role_arn=task_role_arn,
+            execution_role_arn=execution_role_arn,
         )
         self.task_definitions[family][revision] = task_definition
 
@@ -706,38 +786,44 @@ class EC2ContainerServiceBackend(BaseBackend):
 
     def deregister_task_definition(self, task_definition_str):
         task_definition_name = task_definition_str.split("/")[-1]
-        family, revision = task_definition_name.split(":")
-        revision = int(revision)
+        try:
+            family, revision = task_definition_name.split(":")
+        except ValueError:
+            raise RevisionNotFoundException
+        try:
+            revision = int(revision)
+        except ValueError:
+            raise InvalidParameterException(
+                "Invalid revision number. Number: " + revision
+            )
         if (
             family in self.task_definitions
             and revision in self.task_definitions[family]
         ):
-            return self.task_definitions[family].pop(revision)
+            task_definition = self.task_definitions[family].pop(revision)
+            task_definition.status = "INACTIVE"
+            return task_definition
         else:
-            raise Exception("{0} is not a task_definition".format(task_definition_name))
+            raise TaskDefinitionNotFoundException
+
+    def run_task(
+        self, cluster_str, task_definition_str, count, overrides, started_by, tags
+    ):
+        cluster = self._get_cluster(cluster_str)
 
-    def run_task(self, cluster_str, task_definition_str, count, overrides, started_by):
-        if cluster_str:
-            cluster_name = cluster_str.split("/")[-1]
-        else:
-            cluster_name = "default"
-        if cluster_name in self.clusters:
-            cluster = self.clusters[cluster_name]
-        else:
-            raise Exception("{0} is not a cluster".format(cluster_name))
         task_definition = self.describe_task_definition(task_definition_str)
-        if cluster_name not in self.tasks:
-            self.tasks[cluster_name] = {}
+        if cluster.name not in self.tasks:
+            self.tasks[cluster.name] = {}
         tasks = []
         container_instances = list(
-            self.container_instances.get(cluster_name, {}).keys()
+            self.container_instances.get(cluster.name, {}).keys()
         )
         if not container_instances:
-            raise Exception("No instances found in cluster {}".format(cluster_name))
+            raise Exception("No instances found in cluster {}".format(cluster.name))
         active_container_instances = [
             x
             for x in container_instances
-            if self.container_instances[cluster_name][x].status == "ACTIVE"
+            if self.container_instances[cluster.name][x].status == "ACTIVE"
         ]
         resource_requirements = self._calculate_task_resource_requirements(
             task_definition
@@ -745,7 +831,7 @@ class EC2ContainerServiceBackend(BaseBackend):
         # TODO: return event about unable to place task if not able to place enough tasks to meet count
         placed_count = 0
         for container_instance in active_container_instances:
-            container_instance = self.container_instances[cluster_name][
+            container_instance = self.container_instances[cluster.name][
                 container_instance
             ]
             container_instance_arn = container_instance.container_instance_arn
@@ -762,12 +848,13 @@ class EC2ContainerServiceBackend(BaseBackend):
                         resource_requirements,
                         overrides or {},
                         started_by or "",
+                        tags or [],
                     )
                     self.update_container_instance_resources(
                         container_instance, resource_requirements
                     )
                     tasks.append(task)
-                    self.tasks[cluster_name][task.task_arn] = task
+                    self.tasks[cluster.name][task.task_arn] = task
                     placed_count += 1
                     if placed_count == count:
                         return tasks
@@ -851,24 +938,21 @@ class EC2ContainerServiceBackend(BaseBackend):
         overrides,
         started_by,
     ):
-        cluster_name = cluster_str.split("/")[-1]
-        if cluster_name in self.clusters:
-            cluster = self.clusters[cluster_name]
-        else:
-            raise Exception("{0} is not a cluster".format(cluster_name))
+        cluster = self._get_cluster(cluster_str)
+
         task_definition = self.describe_task_definition(task_definition_str)
-        if cluster_name not in self.tasks:
-            self.tasks[cluster_name] = {}
+        if cluster.name not in self.tasks:
+            self.tasks[cluster.name] = {}
         tasks = []
         if not container_instances:
-            raise Exception("No container instance list provided")
+            raise InvalidParameterException("Container Instances cannot be empty.")
 
         container_instance_ids = [x.split("/")[-1] for x in container_instances]
         resource_requirements = self._calculate_task_resource_requirements(
             task_definition
         )
         for container_instance_id in container_instance_ids:
-            container_instance = self.container_instances[cluster_name][
+            container_instance = self.container_instances[cluster.name][
                 container_instance_id
             ]
             task = Task(
@@ -883,17 +967,14 @@ class EC2ContainerServiceBackend(BaseBackend):
             self.update_container_instance_resources(
                 container_instance, resource_requirements
             )
-            self.tasks[cluster_name][task.task_arn] = task
+            self.tasks[cluster.name][task.task_arn] = task
         return tasks
 
     def describe_tasks(self, cluster_str, tasks):
-        cluster_name = cluster_str.split("/")[-1]
-        if cluster_name in self.clusters:
-            cluster = self.clusters[cluster_name]
-        else:
-            raise Exception("{0} is not a cluster".format(cluster_name))
+        self._get_cluster(cluster_str)
+
         if not tasks:
-            raise Exception("tasks cannot be empty")
+            raise InvalidParameterException("Tasks cannot be empty.")
         response = []
         for cluster, cluster_tasks in self.tasks.items():
             for task_arn, task in cluster_tasks.items():
@@ -920,11 +1001,10 @@ class EC2ContainerServiceBackend(BaseBackend):
             for arn, task in tasks.items():
                 filtered_tasks.append(task)
         if cluster_str:
-            cluster_name = cluster_str.split("/")[-1]
-            if cluster_name not in self.clusters:
-                raise Exception("{0} is not a cluster".format(cluster_name))
+            cluster = self._get_cluster(cluster_str)
+
             filtered_tasks = list(
-                filter(lambda t: cluster_name in t.cluster_arn, filtered_tasks)
+                filter(lambda t: cluster.name in t.cluster_arn, filtered_tasks)
             )
 
         if container_instance:
@@ -935,27 +1015,43 @@ class EC2ContainerServiceBackend(BaseBackend):
                 )
             )
 
+        if family:
+            task_definition_arns = self.list_task_definitions(family)
+            filtered_tasks = list(
+                filter(
+                    lambda t: t.task_definition_arn in task_definition_arns,
+                    filtered_tasks,
+                )
+            )
+
         if started_by:
             filtered_tasks = list(
                 filter(lambda t: started_by == t.started_by, filtered_tasks)
             )
+
+        if service_name:
+            # TODO: We can't filter on `service_name` until the backend actually
+            # launches tasks as part of the service creation process.
+            pass
+
+        if desiredStatus:
+            filtered_tasks = list(
+                filter(lambda t: t.desired_status == desiredStatus, filtered_tasks)
+            )
+
         return [t.task_arn for t in filtered_tasks]
 
     def stop_task(self, cluster_str, task_str, reason):
-        cluster_name = cluster_str.split("/")[-1]
-        if cluster_name not in self.clusters:
-            raise Exception("{0} is not a cluster".format(cluster_name))
+        cluster = self._get_cluster(cluster_str)
 
-        if not task_str:
-            raise Exception("A task ID or ARN is required")
         task_id = task_str.split("/")[-1]
-        tasks = self.tasks.get(cluster_name, None)
+        tasks = self.tasks.get(cluster.name, None)
         if not tasks:
-            raise Exception("Cluster {} has no registered tasks".format(cluster_name))
+            raise Exception("Cluster {} has no registered tasks".format(cluster.name))
         for task in tasks.keys():
             if task.endswith(task_id):
                 container_instance_arn = tasks[task].container_instance_arn
-                container_instance = self.container_instances[cluster_name][
+                container_instance = self.container_instances[cluster.name][
                     container_instance_arn.split("/")[-1]
                 ]
                 self.update_container_instance_resources(
@@ -966,7 +1062,7 @@ class EC2ContainerServiceBackend(BaseBackend):
                 tasks[task].stopped_reason = reason
                 return tasks[task]
         raise Exception(
-            "Could not find task {} on cluster {}".format(task_str, cluster_name)
+            "Could not find task {} on cluster {}".format(task_str, cluster.name)
         )
 
     def create_service(
@@ -979,18 +1075,22 @@ class EC2ContainerServiceBackend(BaseBackend):
         scheduling_strategy=None,
         tags=None,
         deployment_controller=None,
+        launch_type=None,
     ):
-        cluster_name = cluster_str.split("/")[-1]
-        if cluster_name in self.clusters:
-            cluster = self.clusters[cluster_name]
-        else:
-            raise Exception("{0} is not a cluster".format(cluster_name))
+        cluster = self._get_cluster(cluster_str)
+
         if task_definition_str is not None:
             task_definition = self.describe_task_definition(task_definition_str)
         else:
             task_definition = None
         desired_count = desired_count if desired_count is not None else 0
 
+        launch_type = launch_type if launch_type is not None else "EC2"
+        if launch_type not in ["EC2", "FARGATE"]:
+            raise InvalidParameterException(
+                "launch type should be one of [EC2,FARGATE]"
+            )
+
         service = Service(
             cluster,
             service_name,
@@ -1000,8 +1100,9 @@ class EC2ContainerServiceBackend(BaseBackend):
             scheduling_strategy,
             tags,
             deployment_controller,
+            launch_type,
         )
-        cluster_service_pair = "{0}:{1}".format(cluster_name, service_name)
+        cluster_service_pair = "{0}:{1}".format(cluster.name, service_name)
         self.services[cluster_service_pair] = service
 
         return service
@@ -1021,29 +1122,42 @@ class EC2ContainerServiceBackend(BaseBackend):
         return sorted(service_arns)
 
     def describe_services(self, cluster_str, service_names_or_arns):
-        cluster_name = cluster_str.split("/")[-1]
+        cluster = self._get_cluster(cluster_str)
+
         result = []
+        failures = []
         for existing_service_name, existing_service_obj in sorted(
             self.services.items()
         ):
             for requested_name_or_arn in service_names_or_arns:
                 cluster_service_pair = "{0}:{1}".format(
-                    cluster_name, requested_name_or_arn
+                    cluster.name, requested_name_or_arn
                 )
                 if (
                     cluster_service_pair == existing_service_name
                     or existing_service_obj.arn == requested_name_or_arn
                 ):
                     result.append(existing_service_obj)
+                else:
+                    service_name = requested_name_or_arn.split("/")[-1]
+                    failures.append(
+                        {
+                            "arn": "arn:aws:ecs:eu-central-1:{0}:service/{1}".format(
+                                ACCOUNT_ID, service_name
+                            ),
+                            "reason": "MISSING",
+                        }
+                    )
 
-        return result
+        return result, failures
 
     def update_service(
         self, cluster_str, service_str, task_definition_str, desired_count
     ):
-        cluster_name = cluster_str.split("/")[-1]
+        cluster = self._get_cluster(cluster_str)
+
         service_name = service_str.split("/")[-1]
-        cluster_service_pair = "{0}:{1}".format(cluster_name, service_name)
+        cluster_service_pair = "{0}:{1}".format(cluster.name, service_name)
         if cluster_service_pair in self.services:
             if task_definition_str is not None:
                 self.describe_task_definition(task_definition_str)
@@ -1054,22 +1168,22 @@ class EC2ContainerServiceBackend(BaseBackend):
                 self.services[cluster_service_pair].desired_count = desired_count
             return self.services[cluster_service_pair]
         else:
-            raise ServiceNotFoundException(service_name)
+            raise ServiceNotFoundException
+
+    def delete_service(self, cluster_name, service_name, force):
+        cluster = self._get_cluster(cluster_name)
+        cluster_service_pair = "{0}:{1}".format(cluster.name, service_name)
 
-    def delete_service(self, cluster_name, service_name):
-        cluster_service_pair = "{0}:{1}".format(cluster_name, service_name)
         if cluster_service_pair in self.services:
             service = self.services[cluster_service_pair]
-            if service.desired_count > 0:
-                raise Exception("Service must have desiredCount=0")
+            if service.desired_count > 0 and not force:
+                raise InvalidParameterException(
+                    "The service cannot be stopped while it is scaled above 0."
+                )
             else:
                 return self.services.pop(cluster_service_pair)
         else:
-            raise Exception(
-                "cluster {0} or service {1} does not exist".format(
-                    cluster_name, service_name
-                )
-            )
+            raise ServiceNotFoundException
 
     def register_container_instance(self, cluster_str, ec2_instance_id):
         cluster_name = cluster_str.split("/")[-1]
@@ -1096,18 +1210,15 @@ class EC2ContainerServiceBackend(BaseBackend):
         return sorted(container_instances)
 
     def describe_container_instances(self, cluster_str, list_container_instance_ids):
-        cluster_name = cluster_str.split("/")[-1]
-        if cluster_name not in self.clusters:
-            raise Exception("{0} is not a cluster".format(cluster_name))
+        cluster = self._get_cluster(cluster_str)
+
         if not list_container_instance_ids:
-            raise JsonRESTError(
-                "InvalidParameterException", "Container instance cannot be empty"
-            )
+            raise InvalidParameterException("Container Instances cannot be empty.")
         failures = []
         container_instance_objects = []
         for container_instance_id in list_container_instance_ids:
             container_instance_id = container_instance_id.split("/")[-1]
-            container_instance = self.container_instances[cluster_name].get(
+            container_instance = self.container_instances[cluster.name].get(
                 container_instance_id, None
             )
             if container_instance is not None:
@@ -1124,14 +1235,12 @@ class EC2ContainerServiceBackend(BaseBackend):
     def update_container_instances_state(
         self, cluster_str, list_container_instance_ids, status
     ):
-        cluster_name = cluster_str.split("/")[-1]
-        if cluster_name not in self.clusters:
-            raise Exception("{0} is not a cluster".format(cluster_name))
+        cluster = self._get_cluster(cluster_str)
+
         status = status.upper()
         if status not in ["ACTIVE", "DRAINING"]:
-            raise Exception(
-                "An error occurred (InvalidParameterException) when calling the UpdateContainerInstancesState operation: \
-                            Container instances status should be one of [ACTIVE,DRAINING]"
+            raise InvalidParameterException(
+                "Container instance status should be one of [ACTIVE, DRAINING]"
             )
         failures = []
         container_instance_objects = []
@@ -1139,7 +1248,7 @@ class EC2ContainerServiceBackend(BaseBackend):
             x.split("/")[-1] for x in list_container_instance_ids
         ]
         for container_instance_id in list_container_instance_ids:
-            container_instance = self.container_instances[cluster_name].get(
+            container_instance = self.container_instances[cluster.name].get(
                 container_instance_id, None
             )
             if container_instance is not None:
@@ -1178,12 +1287,11 @@ class EC2ContainerServiceBackend(BaseBackend):
         container_instance.running_tasks_count += resource_multiplier * 1
 
     def deregister_container_instance(self, cluster_str, container_instance_str, force):
+        cluster = self._get_cluster(cluster_str)
+
         failures = []
-        cluster_name = cluster_str.split("/")[-1]
-        if cluster_name not in self.clusters:
-            raise Exception("{0} is not a cluster".format(cluster_name))
         container_instance_id = container_instance_str.split("/")[-1]
-        container_instance = self.container_instances[cluster_name].get(
+        container_instance = self.container_instances[cluster.name].get(
             container_instance_id
         )
         if container_instance is None:
@@ -1198,28 +1306,24 @@ class EC2ContainerServiceBackend(BaseBackend):
             self.container_instances["orphaned"][
                 container_instance_id
             ] = container_instance
-        del self.container_instances[cluster_name][container_instance_id]
+        del self.container_instances[cluster.name][container_instance_id]
         self._respond_to_cluster_state_update(cluster_str)
         return container_instance, failures
 
     def _respond_to_cluster_state_update(self, cluster_str):
-        cluster_name = cluster_str.split("/")[-1]
-        if cluster_name not in self.clusters:
-            raise Exception("{0} is not a cluster".format(cluster_name))
+        self._get_cluster(cluster_str)
+
         pass
 
     def put_attributes(self, cluster_name, attributes=None):
-        if cluster_name is None or cluster_name not in self.clusters:
-            raise ClusterNotFoundException
+        cluster = self._get_cluster(cluster_name)
 
         if attributes is None:
-            raise JsonRESTError(
-                "InvalidParameterException", "attributes value is required"
-            )
+            raise InvalidParameterException("attributes can not be empty")
 
         for attr in attributes:
             self._put_attribute(
-                cluster_name,
+                cluster.name,
                 attr["name"],
                 attr.get("value"),
                 attr.get("targetId"),
@@ -1298,8 +1402,7 @@ class EC2ContainerServiceBackend(BaseBackend):
         return filter(lambda x: all(f(x) for f in filters), all_attrs)
 
     def delete_attributes(self, cluster_name, attributes=None):
-        if cluster_name is None or cluster_name not in self.clusters:
-            raise ClusterNotFoundException
+        cluster = self._get_cluster(cluster_name)
 
         if attributes is None:
             raise JsonRESTError(
@@ -1308,7 +1411,7 @@ class EC2ContainerServiceBackend(BaseBackend):
 
         for attr in attributes:
             self._delete_attribute(
-                cluster_name,
+                cluster.name,
                 attr["name"],
                 attr.get("value"),
                 attr.get("targetId"),
@@ -1386,7 +1489,7 @@ class EC2ContainerServiceBackend(BaseBackend):
                 if service.arn == resource_arn:
                     return service.tags
             else:
-                raise ServiceNotFoundException(service_name=parsed_arn["id"])
+                raise ServiceNotFoundException
         raise NotImplementedError()
 
     def _get_last_task_definition_revision_id(self, family):
@@ -1403,7 +1506,7 @@ class EC2ContainerServiceBackend(BaseBackend):
                     service.tags = self._merge_tags(service.tags, tags)
                 return {}
             else:
-                raise ServiceNotFoundException(service_name=parsed_arn["id"])
+                raise ServiceNotFoundException
         raise NotImplementedError()
 
     def _merge_tags(self, existing_tags, new_tags):
@@ -1429,13 +1532,13 @@ class EC2ContainerServiceBackend(BaseBackend):
                     ]
                 return {}
             else:
-                raise ServiceNotFoundException(service_name=parsed_arn["id"])
+                raise ServiceNotFoundException
         raise NotImplementedError()
 
     def create_task_set(
         self,
         service,
-        cluster,
+        cluster_str,
         task_definition,
         external_id=None,
         network_configuration=None,
@@ -1448,9 +1551,15 @@ class EC2ContainerServiceBackend(BaseBackend):
         client_token=None,
         tags=None,
     ):
+        launch_type = launch_type if launch_type is not None else "EC2"
+        if launch_type not in ["EC2", "FARGATE"]:
+            raise InvalidParameterException(
+                "launch type should be one of [EC2,FARGATE]"
+            )
+
         task_set = TaskSet(
             service,
-            cluster,
+            cluster_str,
             task_definition,
             self.region_name,
             external_id=external_id,
@@ -1465,16 +1574,14 @@ class EC2ContainerServiceBackend(BaseBackend):
             tags=tags,
         )
 
-        cluster_name = cluster.split("/")[-1]
         service_name = service.split("/")[-1]
 
-        service_obj = self.services.get("{0}:{1}".format(cluster_name, service_name))
+        cluster_obj = self._get_cluster(cluster_str)
+        service_obj = self.services.get(
+            "{0}:{1}".format(cluster_obj.name, service_name)
+        )
         if not service_obj:
-            raise ServiceNotFoundException(service_name=service_name)
-
-        cluster_obj = self.clusters.get(cluster_name)
-        if not cluster_obj:
-            raise ClusterNotFoundException
+            raise ServiceNotFoundException
 
         task_set.task_definition = self.describe_task_definition(task_definition).arn
         task_set.service_arn = service_obj.arn
@@ -1485,21 +1592,18 @@ class EC2ContainerServiceBackend(BaseBackend):
 
         return task_set
 
-    def describe_task_sets(self, cluster, service, task_sets=None, include=None):
+    def describe_task_sets(self, cluster_str, service, task_sets=None, include=None):
         task_sets = task_sets or []
         include = include or []
 
-        cluster_name = cluster.split("/")[-1]
+        cluster_obj = self._get_cluster(cluster_str)
+
         service_name = service.split("/")[-1]
-        service_key = "{0}:{1}".format(cluster_name, service_name)
+        service_key = "{0}:{1}".format(cluster_obj.name, service_name)
 
         service_obj = self.services.get(service_key)
         if not service_obj:
-            raise ServiceNotFoundException(service_name=service_name)
-
-        cluster_obj = self.clusters.get(cluster_name)
-        if not cluster_obj:
-            raise ClusterNotFoundException
+            raise ServiceNotFoundException
 
         task_set_results = []
         if task_sets:
@@ -1549,7 +1653,8 @@ class EC2ContainerServiceBackend(BaseBackend):
             cluster_name, service_name, task_sets=[primary_task_set]
         )[0]
 
-        service_obj = self.describe_services(cluster, [service])[0]
+        services, _ = self.describe_services(cluster, [service])
+        service_obj = services[0]
         service_obj.load_balancers = task_set_obj.load_balancers
         service_obj.task_definition = task_set_obj.task_definition
 
